// $Id: PasswdManager.java,v 1.12 1999/06/28 17:26:11 beca Exp $
/*
File: PasswdManager.java
Component: Server
Date		Author		Changes
Nov 30 98	Lukasz Beca	Created
*/

package webwisdom.tango.server;

 
import java.io.*;
import java.util.*;
import java.security.*;
import webwisdom.tango.test.Log;


/*

PasswdManager class is used to retrieve and write data to server
configuration file. The file has following format:
1. Information about each user is stored in file as one line
2. Line has following structure:
name:password:status:fullname:e-mail
3. All lines that start with # character are treated as comments and
not interpreted by this class

Example:
#comment
luk:4.28.93.-76.-112.-99.26.23.-91.-101.41.32.-77.-49.-22.0.-30.-71.-28.-70:student:Witek Prytek:aaa@bbb.ccc.ddd
#a;dlkfjaldfks
tadeusz:11.24.-68.26.-52.-72.-111.57.-87.-87.52.-83.30.63.-2.-8.-91.-39.76.-49:prof:Wincenty Zielinski:ddd@aaa.hhh.rrr
#wacek i placek
#al;sdkjfsaldkjfaslkdf
placek:110.-70.117.-85.-89.57.4.-63.19.-40.-74.-29.-2.115.-98.-102.77.-34.-26.104:operator:Jozef Tkaczuk:ppp@iii.kkk.kkk
#ha

*/


public class PasswdManager {
  
  private String passwdFilename;
  private MessageDigest md;
  private Hashtable content;
  

  // constant name refers to the information stored on assigned position  
	    
  public static final int NAME = 1;
  public static final int PASSWORD = 2;
  public static final int STATUS = 3;
  public static final int FULLNAME = 4;
  public static final int EMAIL = 5;   
  public static final int ENUMBER = 5;  
  
  
  // constructor: Initializes class with provided filename.
  
  public PasswdManager(String filename) throws IOException {

      passwdFilename = filename;
      
      try { 
       
	      md = MessageDigest.getInstance("SHA");      	       	    
	      
	    } catch(Exception e) {
	    
	      e.printStackTrace(Log.err);
	    
	    } 
	    
	  content = getContent();  
  }

  //close() Ends session with PasswdManager class.
      
  public void close() {
   
    passwdFilename = null;
    md = null;
    content = null;
      	
	}  			
  
  /*getPassword(String name ) Retrieves password of the 'name' user. If no password is
assigned, string of length 0 is returned (not null). If the
user 'name' is not present in the file or entry
format is invalid, null is returned. */
  
  public String getPassword(String name) {
  
      String encPasswd;
            
      encPasswd = get(name, PASSWORD);
                  
      return encPasswd;
  
  }
  
  /*  public String getStatus(String name) 

Retrieves status of the 'name' user. If no status is
assigned, string of length 0 is returned (not null). If the
user 'name' is not present in the file or entry
format is invalid, null is returned.
  */
  
  public String getStatus(String name) {
  
      String status;
            
      status = get(name, STATUS);
                  
      return status;
     
  }
  
  // following methods behave in similar fashion as  getStatus(String name)
  
  public String getFullName(String name) {
  
      String fullName;
            
      fullName = get(name, FULLNAME);
                  
      return fullName;
     
  }
  
  public String getEMail(String name) {
  
      String eMail;
            
      eMail = get(name, EMAIL);
                  
      return eMail;
     
  }
  
  /*
  public Hashtable getPasswordEntries()
  
  Returns Hashtable with all valid password entries in the 
file. Each entry is of the form: (name, encoded_password). 
Name is the key in the hash table. Method returns null if
the password file does not exist.
*/
  
  public Hashtable getPasswordEntries() {
  
    Hashtable result;
    
    result = getEntries(PASSWORD);
      
    return result;

  }

// following methods behave in similar fashion as getPasswordEntries() 
 
  public Hashtable getStatusEntries() {
  
    Hashtable result;
    
    result = getEntries(STATUS);
      
    return result;

  }
  
  public Hashtable getFullNameEntries() {
  
    Hashtable result;
    
    result = getEntries(FULLNAME);
      
    return result;

  }


  public Hashtable getEMailEntries() {
  
    Hashtable result;
    
    result = getEntries(EMAIL);
      
    return result;

  }
  
  
  /*  
  
  public boolean passwordOK(String name, String decPasswd) 
  
     Checks if the password is valid for 'name' user. Password
should be provided in decoded form e.g. "mlotek" or "slon",
not 
"-17.-75.-88.98.-80.101.-98.52.76.91.-111.14.120.69.-22.-68.15.104.59.125"
Returns 'true' if the password is valid, 'false' otherwise. 
    
 */   
  
  public boolean passwordOK(String name, String decPasswd) {
  
      String passwdFromFile;
      String encPasswd;            
      boolean result = false;	    
		        
      passwdFromFile = getPassword(name);
                           
      if (decPasswd == null || passwdFromFile == null) {
      
	       return false;
	
	    } 
           
      if (decPasswd.length() == 0 && passwdFromFile.length() == 0) {
      
	      result = true;
	      
      } else {
      
        encPasswd = getEncPasswd(decPasswd);
	      result = encPasswd.equals(passwdFromFile);
      
      }                   
      
      return result;      
    
  }
  
  /*
    
    public void addEntry(String name, String decPasswd, String status, String fullName, String eMail) throws IOException 
  
Writes user entry in the file. Entry has the format:

name:encoded_password:status

If the entry for 'name' user already exists in the file, the content 
of the entry is overwritten. 
If the entry for 'name' user does not exist, the new entry is added in the
file. 

Special case: it is possible to assign empty ("") password and
status to the user. 

*/
  
  
  public void addEntry(String name, String decPasswd, String status, String fullName, String eMail) throws IOException {
  
  
	   String newLine;
	   String encPasswd;
	   	
	   // check if all fields are valid
	         
     if (name == null || decPasswd == null || status == null || fullName == null || eMail == null) {
     
       return;
     
     }
         
	 	         
	 // get encrypted version of password
		 
     encPasswd = getEncPasswd(decPasswd);          
      
     newLine = name + ":" + encPasswd + ":" + status + ":" + fullName + ":" + eMail;
     		         
	 content.put(name, newLine);
	   								  																																																 																																																																																					 							       																								                                                                         
  }
  
/*
public void removeEntry(String name) throws IOException
  
Removes user entry from the file. Works similar as addEntry();
*/  
  
   public void removeEntry(String name) throws IOException {
  	   
     if (name == null) {
     
       return;
     
     }
     
     content.remove(name);
	 	         																																																																											 							       																								                                                                         
  }

/*

  private String getEncPasswd(String decPasswd) 
  
  Encrypts password


*/

  
  private String getEncPasswd(String decPasswd) {           
         
     String encPasswd;
     StringBuffer tmpPasswd = new StringBuffer();
     byte[] table;
     int i;
         
     md.update(decPasswd.getBytes());                     
     table = md.digest();
     
     for (i = 0; i < table.length - 1; i ++) {
     
       tmpPasswd.append(String.valueOf(table[i]) + ".");     
     
     }
     
     tmpPasswd.append(String.valueOf(table[i]));
               
     encPasswd = tmpPasswd.toString();

     return encPasswd;
  
  }

/*  
      
   public Hashtable getEntries(int column) 

   Returns Hashtable with valid entries of the form: (name, value in column 'column'). 
Name is the key in the hash table. Method returns null if configuration file
does not exit 
      
*/  
  
  public Hashtable getEntries(int column) {
  
    String line, user, data = null;
    Hashtable result = new Hashtable();
    Enumeration names = content.keys();

 
    while(names.hasMoreElements()) {
      
      // read line
      
      user = (String) names.nextElement();                      
      line = (String) content.get(user);	
    
      // find information in the line

      if (line != null) {
      	 		      
		// if not comment line, retrieve name and data stored in column 'column'
		      			  		      			     			      
		data = getField(line, column);				    				    				    												    																    
																												    				    				    
		if (data == null) {
				    
			Log.out.println("PasswdManager: null data at position" + column, 2);
			data = new String("");
		}
				    			    
		// put data in hashtable
							    
		result.put(user, data);
		data = null;				      								  
	  }		         
			    //  System.out.println("PasswdManager: user added with parameters " + user + " " + password);            	    
    } 
        
    return result;
  }
  
  public void update() throws IOException {
    
  
    FileWriter passwdFileWrite;
	BufferedWriter bWriteFile;
	String line, newLine;
	String header1, header2, header3;
	String encPasswd;
	File passwdFile;
	Date today = new Date();
	Enumeration lines = content.elements();
     
    header1 = "# Configuration File for Tango Interactive Collaboration Server";
    header2 = "# Created:" + today;
    header3 = "# This is a generated file. Do not edit!";
     
     // open current config file
     
    passwdFile = new File(passwdFilename);              
     
    // create write stream to the file
     
    passwdFileWrite = new FileWriter(passwdFile);
    bWriteFile = new BufferedWriter(passwdFileWrite);             		                	         
	 
	bWriteFile.write(header1, 0, header1.length()); 
	bWriteFile.newLine();
	bWriteFile.write(header2, 0, header2.length());
	bWriteFile.newLine();
	bWriteFile.write(header3, 0, header3.length());
	bWriteFile.newLine();
	
	while(lines.hasMoreElements()) {    		   		  

        line = (String) lines.nextElement();			      
	    bWriteFile.write(line, 0, line.length());
		bWriteFile.newLine();
		bWriteFile.flush();			
    }						 
					       	  
	// close the file						
	bWriteFile.close();				       
  }


private Hashtable getContent() throws IOException {
  
    String line, user, data = null;
    FileReader passwdFileRead;      
    BufferedReader bufferedFile;    
    File passwdFile;
    Hashtable result = new Hashtable();
  
    // open file for read
  
    try { 
       
       passwdFile = new File(passwdFilename);
       passwdFileRead = new FileReader(passwdFile);     
	    		          
	    
	} catch(Exception e) {
	    
	      passwdFile = null;
	      passwdFileRead = null;
	      //e.printStackTrace(Log.err);	      
	      return result;	    
	}
 
     // create stream for read
 
    bufferedFile = new BufferedReader(passwdFileRead);
 
    do {
      
      // read line
      
      try {
      
        line = bufferedFile.readLine();
	
      } catch(Exception e) {

        Log.out.println("PasswdManager: " + e, 2);
        line = null;
        System.exit(1);
      }
    
      // find information in the line

      if (line != null) {
      
	      if (!line.startsWith("#") && !line.startsWith(" ") && !line.startsWith("\n")){

		      
		      // if not comment line, retrieve name and data stored in column 'column'
		      
			    user = getField(line, NAME);			      			     			      
				data = line;				    				    
				    												    
			    if (user != null) {														    
																												    				    				    
				     if (data == null) {
				    
					     Log.out.println("PasswdManager: line empty", 2);
						 data = new String("");
				     }
				    			    
						// put data in hashtable
							    
				     result.put(user, data);
				     data = null;
				      
				  }
				  
			  }		         
			    //  System.out.println("PasswdManager: user added with parameters " + user + " " + password);
      }               	    

    } while(line != null);
                               
    passwdFileRead.close();   
        
    return result;

  }
  
  


/*

 private String get(String name, int column)
 
Retrieves data of the 'name' user that can be found on 'column' 
position in the line. If no data is specified, string of length 0 is returned (not null). 
If the user 'name' is not present in the file or entry
format is invalid, null is returned.

*/   


  private String get(String name, int column) {
  
    String line, field = null;  
   
    line = (String) content.get(name);
    
    if (line != null) {
    
        field = getField(line, column);	    
    }
    
    return field;          
  }
  
 /*
 private String getField(String line, int column)
 
 retrieves data from the line, stored on position 'column'
     
   
 */ 
  
  
  
  
  private String getField(String line, int column) {
  
    int start, end, pos1, pos2;
    String result = null;
    
    
    pos2 = getPosition(line, ENUMBER - 1);
    
    if (pos2 == -1) 
    
      return null;
    
    
    switch(column) {
    
      case NAME:
	      
	      start = 0;
	      pos1 = getPosition(line, column);
	      if (pos1 != -1) {
	      
		      result = line.substring(start, pos1);
	      
	      }
	      break;
      
      case ENUMBER:
      
	      pos1 = getPosition(line, column - 1);
	      end = line.length();
	      
	      if (pos1 != -1) {
	      
		      result = line.substring(pos1 + 1, end);
	      
	      }
	      break;	      
      
    
      default:
      
	      pos1 = getPosition(line, column - 1);
	      pos2 = getPosition(line, column);
	      
	      if (pos1 != -1 && pos2 != -1) {
	      
		      result = line.substring(pos1 + 1, pos2);
	      
	      }
	                  
    }
    
	
	  return result;      
      
  }
 
  /*
  
  private int getPosition(String line, int occurence)
  
  returns position of i-th occurence of ':' character in the line
  
  
  
  */
   
     
  private int getPosition(String line, int occurence) {
  
    int position = 0;
    int tmp = 0;
  
    while (occurence > 0 && position != -1) {
    
      position = line.indexOf(":", tmp + 1);
      tmp = position;        
      occurence --;
    }
    
    return position;
  
  }
        
}
